Krig.sparse <- function(x,Y,
        cov.fun="expo.cov", cov.fun.args=list(range=10,eps=eps),
        taper.fun="Wu3.cov", taper.fun.args=list(range=10),
	ncov=10000, approxhmax=taper.fun.args$range,		 
        miles=TRUE, R=NULL, eps=10e^-5,
        xM=NULL, m=2,scale.type="user", x.center=rep(0, ncol(x)),
        x.scale=rep(1, ncol(x)),
        maxiter=25, epsiter=10^-3,
        save.sigma=TRUE, save.chol=FALSE, verbose=FALSE,                        
        nnzmax=100000, tmpmax=10000,...){

  if (!is.character(cov.fun)) {
    if (is.function(cov.fun)) {
      cov.fun <- as.character(substitute(cov.fun))
    } else stop("Wrong specification of the covariance")
  }
  if (!is.character(taper.fun)) {
    if (is.function(taper.fun)) {
      taper.fun <- as.character(substitute(taper.fun))
    } else stop("Wrong specification of the taper")
  }
  
  out <- list()
  class(out) <- c("sparse","Krig")
  out$call <- match.call()
  out$timing <- numeric(4)
  names(out$timing) <- c("Covariance","Cholesky","Backsolve","Iterations")
  out$nnz <- numeric(2)
  names(out$nnz) <- c("Covariance","Cholesky")
  out$m <- m
  out$Y <- c(Y)

  if (is.null(cov.fun.args$eps)) cov.fun.args$eps <- eps
  if (is.null(taper.fun.args$eps)) taper.fun.args$eps <- eps
    
  if (is.null(cov.fun.args$nugget))
    out$nugget=0  else   {
      if (cov.fun.args$nugget<0) stop("You should have a positive nugget")
      out$nugget <- cov.fun.args$nugget
      cov.fun.args$nugget <- 0
    }
  out$range <- as.double(taper.fun.args$range)
  
  hlag <- seq(0,to=approxhmax,l=ncov+1)
  out$covapprox <- do.call(cov.fun,c(distance=list(hlag),cov.fun.args))*
    do.call(taper.fun,c(distance=list(hlag),taper.fun.args))

  out$miles <- as.integer(miles)
  if (is.null(R)) R <- 0 
  out$R <- as.double(R)
  out$eps <- as.double(max(eps,10e^-7))

  out$cov.fun <- cov.fun
  out$cov.fun.args <- cov.fun.args
  out$taper.fun <- taper.fun
  out$taper.fun.args <- taper.fun.args
  out$ncov <- as.integer(ncov)

  
  ilen <- as.integer(dim(x)[1])
  
  ### calculate design matrices
  if (is.null(xM))     xM <- x
  
  xM <- transformx(xM, scale.type, x.center, x.scale)
  out$x.center <- attr(xM,"scaled:center")
  out$x.scale <- attr(xM,"scaled:scale")
     
  if (m<0) stop('m, the degree of the polynom should be > 1') else
  if (m==0) stop('You should use simple kriging') else
  if (m==1) {
    xM <- array(1,c(ilen,1))
  } else if (m==2) {
    xn <- colnames(xM)
    if (is.null(xn)) xn <- c('x','y')
    xM <- cbind(xM,1)
    colnames(xM) <- c(xn,'c')
  } else if (m==3) {
    xn <- colnames(xM)
    if (is.null(xn)) xn <- c('x','y')
    xM <- cbind(xM^2,xM[,1]*xM[,2],xM,1)
    colnames(xM) <- c(paste(xn,'.2',sep=''),paste(xn,collapse='.'),xn,'c')
  } else  if (m>3) warning('This degree is not programmed yet')
  out$scale.type <- scale.type
   
    
  out$xM <- xM
  out$invxMtMx <- solve(t(xM)%*%xM)
  rm(xM)
  
  ### calculate covariance matrix
  ptm1 <- proc.time()

  mat <- .Fortran('makesparseappox',
                  ilen,ilen,as.integer(nnzmax),
                  as.double(x[,1]),as.double(x[,2]),
                  as.double(x[,1]),as.double(x[,2]),
                  miles=out$miles,r=out$R,eps=out$eps,
                  covapprox=out$covapprox,ncov=out$ncov,
                  out$range,
                  ira=as.integer(0), ra=as.double(numeric(nnzmax)),
                  ja=as.integer(numeric(nnzmax)),ia=as.integer(numeric(ilen+1)))
  
  out$sigma <- as.matrix.csr(1,1,1)
  slot(out$sigma,"ra") <- mat$ra[1:mat$ira]
  slot(out$sigma,"ja") <- as.integer(mat$ja[1:mat$ira])
  slot(out$sigma,"ia") <- as.integer(mat$ia)
  slot(out$sigma,"dimension") <- as.integer(c(ilen,ilen))
  slot(out$sigma,"ra") <- .Fortran('addnugget',out$nugget,
            as.integer(out$nnz[1]), as.integer(ilen),                
            ra=slot(out$sigma,"ra"),slot(out$sigma,"ja"),slot(out$sigma,"ia"))$ra

  out$nnz[1] <- mat$ira 
  rm(mat)
  out$timing[1] <- (ptm1 <- proc.time()-ptm1)[3]
  if (verbose) cat('\n\n  Covariance  :',out$timing[1],'sec')

  ### calculate cholesky
  ptm1 <- proc.time()
  sigmachol <- chol(out$sigma, tmpmax=tmpmax, ...)
  out$nnz[2] <- sigmachol@nnzl
  out$timing[2] <- (ptm1 <- proc.time()-ptm1)[3]
  if (verbose) cat('\n  Cholesky    :',out$timing[2],'sec')

  out$sigmanonu <- out$sigma
  slot(out$sigmanonu,"ra") <- .Fortran('addnugget',-out$nugget,
            as.integer(out$nnz[1]), as.integer(ilen),                
            ra=slot(out$sigma,"ra"),slot(out$sigma,"ja"),slot(out$sigma,"ia"))$ra
 
  ### start iterate...
  ptm1 <- proc.time()
  out$iternorm <- numeric(maxiter)
  doiteration <- TRUE
  nriteration <- 1
  old.coef <- 0
  spatial <- 0
  
  while (doiteration) {


    coef <- c( (out$Y - spatial) %*%  out$xM %*%  out$invxMtMx)
    
    trend <- c(out$xM %*% coef)
    spatial <- c(out$sigmanonu %*% backsolve(sigmachol, (out$Y - trend)))
    
    out$iternorm[nriteration] <- mean((coef-old.coef)^2)
    if(verbose)cat('\n          Normdifference : ',out$iternorm[nriteration])
    if(verbose)cat('\n          Coefficients   : ',coef)
 
    if ((nriteration==maxiter)| (out$iternorm[nriteration] < epsiter)){
      doiteration <- FALSE
    } else {
      nriteration <- nriteration + 1 
      old.coef <- coef
    }
  
    NULL
  }
  out$timing[4] <- (ptm1 <- proc.time()-ptm1)[3]
  if (verbose) cat('\n  Iterations  :',out$timing[4],'sec\n\n')

  out$iternorm <- out$iternorm[1:nriteration]
  out$iterpar <- c(maxiter,epsiter)
  out$coef <- coef
  out$spatial <- spatial
  out$trend <- trend
  ptm1 <- proc.time()
  out$solve <- backsolve(sigmachol, (out$Y - trend))
  out$timing[3] <- (ptm1 <- proc.time()-ptm1)[3]

  out$fitted <- out$trend + out$spatial
  ### end iterate
  
  if (save.chol)  {  # construt a good Cholesky factor
    invp <- slot(sigmachol,"invp")
    jaF <- .Fortran('calcja',
                as.integer(obs.len),as.integer(sigmachol@nnzl),
                as.integer(sigmachol@nnzlindx),as.integer(sigmachol@nsuper),
                as.integer(sigmachol@xsuper),as.integer(sigmachol@lindx),
                as.integer(sigmachol@xlindx),as.integer(sigmachol@xlnz),
                cholcja=as.integer(numeric(sigmachol@nnzl)))
  

    out$sigmachol <- as.matrix.csr(1,1,1)
    slot(out$sigmachol,"ra") <- slot(sigmachol,"lnz")
    slot(out$sigmachol,"ja") <- jaF$cholcja
    slot(out$sigmachol,"ia") <- as.integer(sigmachol@xlnz)
    slot(out$sigmachol,"dimension") <- as.integer(c(obs.len,obs.len))
  }    

  if (!save.sigma) out$sigma <- NULL
  out$Y <- as.double(Y) 
  out$x <- x
  

  invisible(out)
}

