covapprox.error <- function(n,cov.fun='expo.cov',cov.fun.args=list(range=1),
                            taper.fun='Wu3.cov',taper.fun.args=list(range=1),
                            hmax=taper.fun.args$range,nres=25) {

  if (!is.character(cov.fun)) {
    if (is.function(cov.fun)) {
      cov.fun <- as.character(substitute(cov.fun))
    } else stop("Wrong specification of the covariance")
  }
  if (!is.character(taper.fun)) {
    if (is.function(taper.fun)) {
      taper.fun <- as.character(substitute(taper.fun))
    } else stop("Wrong specification of the taper")
  }
  
  h <- hmax/n
  hlag <- seq(0,to=hmax,l=n+1)
  covhlag <- do.call(cov.fun,c(distance=list(hlag),cov.fun.args))*
    do.call(taper.fun,c(distance=list(hlag),taper.fun.args))

  hlagres <- seq(0,to=hmax,l=n*nres+1)
  covhlagres <- do.call(cov.fun,c(distance=list(hlagres),cov.fun.args))*
    do.call(taper.fun,c(distance=list(hlagres),taper.fun.args))

  m <- diff(covhlag)/h
  hrep <- rep(hlag[-n],rep(nres,n))
  frep <- rep(covhlag[-n],rep(nres,n))
  mrep <- rep(m,rep(nres,n))
  
  covapp <- mrep*(hlagres[-n*nres+1]-hrep) + frep
  error <- covhlagres[-n*nres+1] - covapp
  invisible(list(max=max(abs(error)),ise=sum(error^2)*h/nres,
                 iae=sum(abs(error))*h/nres,
                 covapprox=covapp,error=error))
}

