\name{Krig.simple.sparse}
\alias{Krig.simple.sparse}
\title{
  Kriging surface estimate  
}
\description{Calculating the BLUP or kriging estimate for
  large two dimensional spatial datasets.
}
\usage{Krig.simple.sparse(x,Y,
        cov.fun = "expo.cov", cov.fun.args = list(range = 10, 
        eps = eps), taper.fun = "Wu3.cov", taper.fun.args = list(range = 10), 
        ncov = 10000, approxhmax = taper.fun.args$range, miles = TRUE, 
        R = NULL, eps = 10^-5, save.sigma = TRUE, save.chol = FALSE, 
        verbose = FALSE, nnzmax = 1e+05, tmpmax = 10000, ...) }
\arguments{
 \item{x}{a \code{m} times \code{2} matrix containing the locations.}
 \item{Y}{the observed values at \code{x}.}

 \item{cov.fun}{Covariance function in the form of an \R function, or its
       	name as a string.}
 \item{cov.fun.args}{A list with the arguments to call the covariance function
 	(in addition to the locations).}
 \item{taper.fun}{Taper function in the form of an \R function, or its
	name as a string.}
 \item{taper.fun.args}{A list with the arguments to call the taper function
          (in addition to the locations).}
 \item{ncov}{Number of knots to evaluate the approximation}
 \item{approxhmax}{Maximum distance over which the covariance is approximated}

 \item{miles}{logical. If \code{TRUE} (default) distances are in statute
   miles if \code{FALSE} distances in kilometers.}  
 \item{R}{the radius to use for the sphere to find spherical
   distances. If \code{NULL} the radius is either in miles or kilometers
   of the earth depending on the values of the miles argument. If
   \code{R=1} then distances are in radians.}
 \item{eps}{small value, everything smaller is considered zero.}

 \item{save.sigma}{should the covariance matrix be saved (in
   \code{SparseM} format).} 
 \item{save.chol}{should the Cholesky factor of the covariance matrix be
   saved (in \code{SparseM} format).}
 \item{verbose}{should timing and convergence results be printed.}
 \item{nnzmax}{upper bound of non-zero elements in the covariance matrix.}

 \item{tmpmax}{working array for the Cholesky factorisation}
 \item{...}{supplementary parameters that can be given as arguments to
   the function \code{chol}.}
}
\value{\code{Krig.simple.sparse} returns an object of \code{class}
  \code{c("sparse","Krig")}. The second is to reuse many
  handy functions of the library \code{fields}.

An object of the class \code{"sparse"} is a list containing at
least the following components:
 \item{call}{the matched call.}

 \item{fitted}{fitted values at observed values.}
 \item{solve}{vector used for prediction on other grid points.}
 \item{sigma}{if requested, the covariance matrix.}
 \item{sigmachol}{if requested, the Cholesky factor.}
 \item{timing}{time needed for the main calculations.}
 \item{nnz}{The number of nonzero elements in the covariance matrix and
   its Cholesky factor.}

 Additionally, most input arguments are passed to the object.
}
\details{
  For computational reasons, we do not call simple the \code{solve}
  function but use \code{chol(...)} and \code{backsolve(...)} form the
  \code{SparseM} library.
  We do not allow missing values.
  We only consider two-dimensional domains.

}
\note{For REALLY big datasets, it would be wise to dissect the
  functions.

  The radius of the earth is assumed to be 3963.34 miles or 6378.388
  kilometers. 

  \code{approxhmax} should be at least as big as the taper range or 
	the domain of the field.

  If \code{nnzmax} is too small, \R may produce a `core dumped'.
}
\seealso{\code{\link{Krig.simple.sparse}},
  \code{\link{predict.sparse}}, \code{\link{plot.sparse}};
  
  \code{chol} and \code{backsolve} from the \code{SparseM} library.}
\examples{data(simple)
attach(simple.data)

obj <- Krig.simple.sparse( x, Y)
pre <- predict( obj, x=cbind(-104.5,40.5))

surf <- predict.surface( obj)
image.plot( surf)

}
\keyword{spatial}


