\name{covapprox.error}
\alias{covapprox.error}
\title{Error of linear covariance approximation}
\description{Evaluates different error measures of the linear
covariance approximation used in the kriging approach}
\usage{covapprox.error(n, cov.fun='expo.cov', cov.fun.args=list(range=1),
          taper.fun='Wu3.cov', taper.fun.args=list(range=1),
          hmax=taper.fun.args$range,nres=25)}
\arguments{
 \item{n}{Number of knots to evaluate the approximation}
 \item{cov.fun}{Covariance function in the form of an \R function, or its
       	name as a string.}
 \item{cov.fun.args}{A list with the arguments to call the covariance function
 	(in addition to the locations).}
 \item{taper.fun}{Taper function in the form of an \R function, or its
	name as a string.}
 \item{taper.fun.args}{A list with the arguments to call the taper function
          (in addition to the locations).}
 \item{hmax}{Distance over which the error is calculated.}
 \item{nres}{Resolution over which the errors are calculated.}
}
\details{\code{n=1000} usually gives a maximum error smaller than 0.1
   percent of the total sill.}
\value{A list with the elements:
 \item{max}{Maximum of the error.}
 \item{ise}{Approximation of the integrated squared error.}
 \item{iae}{Approximation of the integrated absolute error.}
 \item{covapprox}{Linear approximation of the covariance.}
 \item{error}{Error committed using the linear interpolation.}}
\note{To obtain a `small' tapering effect, the taper range
	can be set to a large value. In such a case, \code{hmax} should
	be set to the diameter of the domain. 

	Using the \code{tophat} function as taper, no tapering
	is done.}
\seealso{Covariance functions such as \code{\link{expo.cov}},
  \code{\link{mater.cov}}, etc.
  
	\code{\link{Krig.sparse}} and
	\code{\link{Krig.simple.sparse}}.}
\examples{
# plot the error using the default functions.
nres <- 10
n <- 25
h <- seq(0,to=1, l=n*nres)
plot( h, covapprox.error(n,nres=nres)$error, type='l')


# evaluate error for a covariance only.
expoapprox <- covapprox.error(1000,taper.fun=tophat,
               taper.fun.args=list(range=5))
# all values are negative, as expo.cov is convex.
}
\keyword{spatial}


