\name{covariance}
\alias{expo.cov}
\alias{mater.cov}
\alias{spher.cov}
\alias{tri.cov}
\alias{Wu1.cov}
\alias{Wu2.cov}
\alias{Wu3.cov}
\alias{tophat}
\title{Theoretical Distance Based Covariance Functions}
\description{Computes theoretical covariance function at supplied
distance values. Models include exponential, spherical, Matern
and compactly supported covariances.
}
\usage{spher.cov(distance, range, sill=1, nugget=0, eps=1.0e-7,...)
expo.cov(distance, range, sill=1, nugget=0, effect=T, eps=1.0e-7,...)
mater.cov(distance, smooth, range, sill=1, nugget=0, eps=1.0e-7,...)
tri.cov(distance, range, sill=1, nugget=0, eps=1.0e-7,...)
Wu1.cov(distance, range, sill=1, nugget=0, eps=1.0e-7,...)
Wu2.cov(distance, range, sill=1, nugget=0, eps=1.0e-7,...)
Wu3.cov(distance, range, sill=1, nugget=0, eps=1.0e-7,...)
tophat(distance, range, sill=1,...)
}
\arguments{
 \item{distance}{a vector/matrix of distances to compute the covariance for.}
 \item{range}{the range value.}
 \item{smooth}{smoothness of the Matern covariance.}
 \item{sill}{the partial sill value. The absolute sill (the
   variance or equivalently the covariance at distance zero) is
   \code{sill + nugget}.}
 \item{nugget}{the nugget effect.}
 \item{effect}{if \code{TRUE}, \code{range} is the practial range}
 \item{eps}{any distance less than it will be set to \code{nugget +
     sill}.}
 \item{...}{see below.}
}
\value{a vector/matrix of covariance values at the supplied distances.
}
\details{The tophat is NOT an actual covariance function. It is included
  for illustration purposes only.

  The triangular, spherical and the Wu type functions are compactly
  supported covariance functions, vanishing beyond the range. The triangular 
  and the first Wu type are not valid in two dimensions and up.}

\note{To all the covariance functions the \code{...} argument is added
  to ensure compatibility between different types of covariance
  functions. Although this would not be necessary, it simplifies
  internal coding and usage considerably.
  
  There is a difference between \code{exp.cov} and \code{expo.cov}, as well as
  \code{matern.cov} and \code{mater.cov} functions in \code{fields} and \code{KriSp}.
}
\examples{distance <- seq(0,2,length=150)
plot( distance, mater.cov(distance,smooth=1,range=.4,sill=.8,nugget=.2))
}
\keyword{spatial}


