\name{datasets}
\alias{simple.data}
\alias{simple}
\alias{universal.data}
\alias{universal}
\title{
  Artificial datasets used in the tutorial 
}
\description{
The \code{simple.data} and \code{universal.data} are artifical datasets
to illustrate the simple and universal kriging in the tutorial.
}
\usage{
data(simple)
data(universal)
}
\format{
A list containing the two components.
  \code{x}: 100 longitude-latitude position of locations,
  \code{Y}: the simulated values.
}
\seealso{\code{\link{anomaly.data}}.}
\examples{
library(mvtnorm)

# simple.data:
set.seed(15)
n <- 100
x <- round(cbind(lon=runif(n)-105,lat=runif(n)+40),3)

lags <- rdist.earth(x)
C <- expo.cov( lags, range=10, sill=1)

Y <- round(c(rmvnorm(1,sigma=C)), 3)
simple.data <- list(x=x,Y=Y)


# universal.data:
set.seed(15)
n <- 100
x <- round(cbind(lon=runif(n)-105,lat=runif(n)+40),3)

lagC <- rdist.earth(x)
C <- expo.cov( lagC, range=10, sill=0.9, nugget=0.1)

Y <- round(c(rmvnorm(1,sigma=C))+4*x[,2]-100, 3)
universal.data <- list(x=x,Y=Y)
}
\keyword{datasets}
