\name{predict.sparse}
\alias{predict.sparse}
\title{Evaluation of Krig spatial process estimate}
\description{Provides predictions from the spatial process estimate at
     arbitrary points}
\usage{predict.sparse(object, x=NULL, trend.only=FALSE, ...)}
\arguments{
 \item{object}{an object of class \code{sparse}, typically the result of a call to
          \code{Krig.sparse} or \code{Krig.simple.sparse}.}
 \item{x}{Matrix of x-values on which to evaluate the kriging surface. 
          If omitted, the data x-values, i.e. \code{obj$x}  will be used.}
 \item{trend.only}{for universal kriging, should only the trend be returned.}
 \item{...}{only for compatibility reasons.}
}
\details{We evaluate the kriging surface on the given grid.}
\value{Vector of predicted responses.}

\seealso{\code{\link{Krig.sparse}}, \code{\link{Krig.sparse}}; 
  \code{predict.surface} from the \code{fields} package.}
\examples{data(universal)
attach(universal.data)

obj <- Krig.sparse(x, Y,
          cov.fun.args=list(range=10,sill=.9,nugget=.1))

print( predict( obj, x=cbind(-104.5,40.5)))

xgrid <- make.surface.grid( grid.list=list(lon='x',lat='y'),
                           X=x, nx=50, ny=50)
surf <- predict( obj, x=xgrid)


image.plot( predict.surface(obj))
image.plot( predict.surface(obj, trend.only=TRUE))
}
\keyword{spatial}


