      SUBROUTINE ADDNUGGET(nugget,nnz,ilen,ra,ja,ia)

c     INPUT/OUTPUT:
c     nugget (double) to add
c     nnz (integer) number of nonzero elements
c     ilen (integer) number of columns 
c     ra,ja,ia (double,integer,integer) ra is overwritten!!

      IMPLICIT NONE

      DOUBLE PRECISION nugget

      INTEGER nnz,ilen,i,j
      
      DOUBLE PRECISION ra(nnz)
      INTEGER ia(ilen+1),ja(nnz)
     

      DO i=1,ilen 
         j=0
 10      IF (ja(ia(i)+j) .lt. i) THEN
            j=j+1
            GOTO 10
         ENDIF
         ra(ia(i)+j)=ra(ia(i)+j)+nugget
      ENDDO

      RETURN
      END


      
      
      SUBROUTINE PREDICTVECTOR(ilen,jlen,x1,x2,y1,y2,v,miles,r,
     %     eps,range,
     %     z)

c     INPUT:
c     x1,x2,y1,y2 (double) coordinates
c     ilen,jlen (integer) number of x- and y-points
c     v (double) vector
c     r (double) radius of sphere, if 0 will be set to earth radius
c     miles (integer), if 1 we assume miles
c     range (double) maximum range
c
c     OUTPUT:
c     z (double) result of matrix multiplication

      IMPLICIT NONE 
      
      INTEGER i,j,ilen,jlen,miles 
      DOUBLE PRECISION x1(ilen),x2(ilen),y1(jlen),y2(jlen) 
      DOUBLE PRECISION v(ilen)
      DOUBLE PRECISION cosx1,sinx1,cosx2,sinx2
      DOUBLE PRECISION cosy1(jlen),siny1(jlen),cosy2(jlen),siny2(jlen) 
      DOUBLE PRECISION r,rad 
      DOUBLE PRECISION res,sigma
      DOUBLE PRECISION eps,range,invtap

      DOUBLE PRECISION z(ilen)


      invtap=1./range
      rad=0.01745329   ! pi/180

      IF (r .le. 0.0001) THEN
         IF (miles .eq. 1) THEN
            r=3963.34
         ELSE
            r=6378.388
         ENDIF
      ENDIF

      DO j=1,jlen
         y1(j)=y1(j)*rad
         y2(j)=y2(j)*rad
         cosy1(j)=cos(y1(j))
         siny1(j)=sin(y1(j))
         cosy2(j)=cos(y2(j))
         siny2(j)=sin(y2(j))         
      ENDDO

c     for each value in x
      DO i=1,ilen
         x1(i)=x1(i)*rad
         x2(i)=x2(i)*rad
         cosx1=cos(x1(i))
         sinx1=sin(x1(i))
         cosx2=cos(x2(i))
         sinx2=sin(x2(i))         

         z(i)=0
c     for each value in y
         DO j=1,jlen
            res=( cosx1*cosx2 ) * ( cosy1(j)*cosy2(j) )
     %        + ( sinx1*cosx2 ) * ( siny1(j)*cosy2(j) )
     %        + sinx2*siny2(j)
            IF (res .gt. 1) THEN
               res=0
            ELSE
               res=r*acos(res)   
            ENDIF
            
            IF (res .le. range) THEN
              CALL TAPER(res,sigma,invtap) 
              IF (sigma .ge. eps) THEN
                 z(i)=z(i)+sigma*v(j)
              ENDIF
           ENDIF

        ENDDO
      ENDDO

      RETURN
      END

      SUBROUTINE PREDICTVECTORAPPROX(ilen,jlen,x1,x2,y1,y2,v,miles,r,
     %     eps,covapprox,ncov,taperlen,nugget,
     %     z)

c     INPUT:
c     x1,x2,y1,y2 (double) coordinates
c     ilen,jlen (integer) number of x- and y-points
c     v (double) vector
c     r (double) radius of sphere, if 0 will be set to earth radius
c     miles (integer), if 1 we assume miles
c     taperlen (double) maximum range
c
c     OUTPUT:
c     z (double) result of matrix multiplication

      IMPLICIT NONE 
      
      INTEGER i,j,ilen,jlen,miles,itmp,ncov 
      DOUBLE PRECISION x1(ilen),x2(ilen),y1(jlen),y2(jlen) 
      DOUBLE PRECISION v(ilen)
      DOUBLE PRECISION cosx1,sinx1,cosx2,sinx2
      DOUBLE PRECISION cosy1(jlen),siny1(jlen),cosy2(jlen),siny2(jlen) 
      DOUBLE PRECISION r,rad 
      DOUBLE PRECISION res,sigma
      DOUBLE PRECISION eps,taperlen,covapprox(ncov),invtap,nugget

      DOUBLE PRECISION z(ilen)


      rad=0.01745329   ! pi/180

      IF (r .le. 0.0001) THEN
         IF (miles .eq. 1) THEN
            r=3963.34
         ELSE
            r=6378.388
         ENDIF
      ENDIF

      DO j=1,jlen
         y1(j)=y1(j)*rad
         y2(j)=y2(j)*rad
         cosy1(j)=cos(y1(j))
         siny1(j)=sin(y1(j))
         cosy2(j)=cos(y2(j))
         siny2(j)=sin(y2(j))         
      ENDDO

c     for each value in x
      DO i=1,ilen
         x1(i)=x1(i)*rad
         x2(i)=x2(i)*rad
         cosx1=cos(x1(i))
         sinx1=sin(x1(i))
         cosx2=cos(x2(i))
         sinx2=sin(x2(i))         

         z(i)=0
c     for each value in y
         DO j=1,jlen
            res=( cosx1*cosx2 ) * ( cosy1(j)*cosy2(j) )
     %        + ( sinx1*cosx2 ) * ( siny1(j)*cosy2(j) )
     %        + sinx2*siny2(j)
            IF (res .gt. 1) THEN
               res=0
            ELSE
               res=r*acos(res)   
            ENDIF
            
            IF (res .le. taperlen) THEN
c 	exact call
c              CALL TAPER(res,sigma,invtap) 
c	approximative call:
               IF (res .le. eps) THEN
                  sigma=covapprox(1)+nugget
               ELSE
                  itmp=res/taperlen*ncov+1
                  sigma=(covapprox(itmp)+covapprox(itmp+1))/2.
               ENDIF
               IF (sigma .ge. eps) THEN
                  z(i)=z(i)+sigma*v(j)
              ENDIF
           ENDIF

        ENDDO
      ENDDO

      RETURN
      END

      SUBROUTINE MAKESPARSE(ilen,jlen,ijlen,x1,x2,y1,y2,miles,r,
     %     eps,range,
     %     ira,ra,ja,ia)

c     INPUT:
c     x1,x2,y1,y2 (double) coordinates
c     ilen,jlen (integer) number of x- and y-points
c     ijlen (integer) max space (ra,ja and ia dim)
c     r (double) radius of sphere, if 0 will be set to earth radius
c     miles (integer), if 1 we assume miles
c     range (double) maximum range
c
c     OUTPUT:
c     ira, (integer) nzz, i.e. number of nonzero elements
c     ra,ja,ia (double,integer,integer) 

      IMPLICIT NONE 
      
      INTEGER ilen,jlen,ijlen,miles 
      DOUBLE PRECISION x1(ilen),x2(ilen),y1(jlen),y2(jlen) 
      DOUBLE PRECISION cosx1,sinx1,cosx2,sinx2
      DOUBLE PRECISION cosy1(jlen),siny1(jlen),cosy2(jlen),siny2(jlen) 
      DOUBLE PRECISION r,rad 
      DOUBLE PRECISION res
      DOUBLE PRECISION eps,range,invtap

      INTEGER ira,i,j
      
      DOUBLE PRECISION ra(ijlen)
      INTEGER iia,ia(ilen+1),ja(ijlen)

      invtap=1./range
      rad=0.01745329   ! pi/180

      IF (r .le. 0.0001) THEN
         IF (miles .eq. 1) THEN
            r=3963.34
         ELSE
            r=6378.388
         ENDIF
      ENDIF

      DO j=1,jlen
         y1(j)=y1(j)*rad
         y2(j)=y2(j)*rad
         cosy1(j)=cos(y1(j))
         siny1(j)=sin(y1(j))
         cosy2(j)=cos(y2(j))
         siny2(j)=sin(y2(j))         
      ENDDO

      ira=1
      ia(1)=1

c     for each value in x
      DO i=1,ilen
         x1(i)=x1(i)*rad
         x2(i)=x2(i)*rad
         cosx1=cos(x1(i))
         sinx1=sin(x1(i))
         cosx2=cos(x2(i))
         sinx2=sin(x2(i))         

         iia=0
c     for each value in y
         DO j=1,jlen
            res=( cosx1*cosx2 ) * ( cosy1(j)*cosy2(j) )
     %        + ( sinx1*cosx2 ) * ( siny1(j)*cosy2(j) )
     %        + sinx2*siny2(j)
            IF (res .gt. 1) THEN
               res=0
            ELSE
               res=r*acos(res)   
            ENDIF
            
            IF (res .le. range) THEN
              CALL TAPER(res,ra(ira),invtap) 
              IF (ra(ira) .ge. eps) THEN  ! if value smaller than eps
                 ja(ira)=j                ! we do not taper
                 
                 ira=ira+1
                 iia=iia+1
              ENDIF
           ENDIF

        ENDDO
         
        ia(i+1)=ia(i)+iia
      
      ENDDO
      ira=ira-1

      RETURN
      
      END
      
      SUBROUTINE MAKESPARSEAPPOX(ilen,jlen,ijlen,x1,x2,y1,y2,
     %     miles,r,eps,covapprox,ncov,taperlen,
     %     ira,ra,ja,ia)

c     INPUT:
c     x1,x2,y1,y2 (double) coordinates
c     ilen,jlen (integer) number of x- and y-points
c     ijlen (integer) max space (ra,ja and ia dim)
c     r (double) radius of sphere, if 0 will be set to earth radius
c     miles (integer), if 1 we assume miles
c     range (double) maximum range
c
c     OUTPUT:
c     ira, (integer) nzz, i.e. number of nonzero elements
c     ra,ja,ia (double,integer,integer) 

      IMPLICIT NONE 
      
      INTEGER ilen,jlen,ijlen,miles, ncov
      DOUBLE PRECISION x1(ilen),x2(ilen),y1(jlen),y2(jlen) 
      DOUBLE PRECISION cosx1,sinx1,cosx2,sinx2
      DOUBLE PRECISION cosy1(jlen),siny1(jlen),cosy2(jlen),siny2(jlen) 
      DOUBLE PRECISION r,rad 
      DOUBLE PRECISION res,invtap
      DOUBLE PRECISION eps,taperlen,covapprox(ncov)

      INTEGER ira,i,j,itmp
      
      DOUBLE PRECISION ra(ijlen)
      INTEGER iia,ia(ilen+1),ja(ijlen)

      invtap=1./taperlen
      rad=0.01745329   ! pi/180

      IF (r .le. 0.0001) THEN
         IF (miles .eq. 1) THEN
            r=3963.34
         ELSE
            r=6378.388
         ENDIF
      ENDIF

      DO j=1,jlen
         y1(j)=y1(j)*rad
         y2(j)=y2(j)*rad
         cosy1(j)=cos(y1(j))
         siny1(j)=sin(y1(j))
         cosy2(j)=cos(y2(j))
         siny2(j)=sin(y2(j))         
      ENDDO

      ira=1
      ia(1)=1

c     for each value in x
      DO i=1,ilen
         x1(i)=x1(i)*rad
         x2(i)=x2(i)*rad
         cosx1=cos(x1(i))
         sinx1=sin(x1(i))
         cosx2=cos(x2(i))
         sinx2=sin(x2(i))         

         iia=0
c     for each value in y
         DO j=1,jlen
            res=( cosx1*cosx2 ) * ( cosy1(j)*cosy2(j) )
     %        + ( sinx1*cosx2 ) * ( siny1(j)*cosy2(j) )
     %        + sinx2*siny2(j)
            IF (res .gt. 1) THEN
               res=0
            ELSE
               res=r*acos(res)   
            ENDIF
            
            IF (res .le. taperlen) THEN
c     original call for exact tapers would be:
c              CALL TAPER(res,ra(ira),invtap) 
c     end block for original call for exact tapers
               IF (res .le. eps) THEN
                  ra(ira)=covapprox(1)
               ELSE
                  itmp=res/taperlen*ncov+1
                  ra(ira)=(covapprox(itmp)+covapprox(itmp+1))/2.
               ENDIF
               ja(ira)=j   
                  
               ira=ira+1
               iia=iia+1               
            ENDIF

         ENDDO
         
         ia(i+1)=ia(i)+iia
      
      ENDDO
      ira=ira-1

      RETURN
      
      END
      
      SUBROUTINE TAPER(res,sol,invtap)
c     example for an exact taper subroutine
c
c     is specific, only taper range is parameterized
c     no nugget is included!!
c     factor is 5

c     INPUT: scalar res, invtap
c     OUTPUT: scalar sol
      
      IMPLICIT NONE 
      
      DOUBLE PRECISION res,sol
      DOUBLE PRECISION invtap,range1,range2  ! inverses !!
      DOUBLE PRECISION sill1,sill2,dis

      range1=0.1221598       !  1/(40.930/rfac)
      range2=0.009474542     ! 1/(527.73/rfac)
      sill1=0.277
      sill2=0.722
      dis=res*invtap

      sol=5 * (1 - 1.5*dis + 0.5*(dis)**3)
     %     * ( sill1*exp(-res*range1) + sill2*exp(-res*range2) )

      RETURN
      END
      
      


      SUBROUTINE CALCJA(nrow,nnz,nnzlindx,nsuper,
     %     xsuper,lindx,xlindx,xlnz,
     %     cholcja)
c     small function to calculate ja for the cholesky factor
c     as they use a condensed format. GRATULIERU LIT!


c     INPUT:
c     nnz (integer)            number of non zero elements
c     nnzlindx (integer)       number of condensed ja elements
c     nrow (integer)           number of rows
c     nsuper (integer)         number of supernodes
c     xsuper (integer)         supernode partition
c     xlindx,lindx  (integer)  row indices for each supernode
c     xlnz (integer)           ia for cholesky factor
c
c     OUTPUT:
c     cholcja (integer)        ja for cholesky factor

      IMPLICIT NONE 
      
      INTEGER nnz,nrow,nsuper,nnzlindx
      INTEGER xsuper(nrow),lindx(nnzlindx),xlindx(nrow+1),xlnz(nrow+1)
      INTEGER cholcja(nnz)

      INTEGER k, i, j, m, n 
      
      k=1
      m=1
      DO i=1,nsuper
         DO j=1,( xsuper(i+1)-xsuper(i))
            DO n=1,(xlnz(k+1)-xlnz(k))
               cholcja(m)=lindx( xlindx(i)+j-2  + n)
               m=m+1
            ENDDO
            k=k+1
         ENDDO
      ENDDO
      
      RETURN
      END      
